<template>
  <van-popup v-model="visible" position="bottom" :style="{ height: '100%' }" @opened="keepMessageListOnButtom">
    <van-nav-bar :title="nick" left-arrow @click-left="onClickLeft" />
    <div class="main">
      <!-- 对话主体 -->
      <div class="conversation">
        <div class="content" :style="contentHeight" ref="message-list" @touchstart="showMore = false">
          <message-item v-for="(message, index) in currentMessageList" :key="message.ID + '-' + index" :message="message" />
        </div>
        <div class="footer">
          <i v-if="chatType === 'text'" class="chat-type chat-type-voice" @click="chatType = 'voice'"></i>
          <i v-else class="chat-type chat-type-text" @click="chatType = 'text'"></i>
          <input
            v-if="chatType === 'text'"
            v-model="message"
            class="chat-input"
            type="text"
            placeholder="说点什么呢～"
            enterkeyhint="send"
            @keyup.enter="send"
            @focus="showMore = false"
          />
          <div v-if="chatType === 'voice'" class="voice-block" @touchstart="startRecording" @touchend="endRecording">
            <span v-if="!recording">按住 说话</span>
            <span v-if="recording">松开发送</span>
          </div>
          <i class="btn btn-emoji" @click="clickShowMore('emoji')"></i>
          <i class="btn btn-more" @click="clickShowMore('file')"></i>
        </div>
        <div v-if="showMore" class="under-footer">
          <div v-if="moreType === 'emoji'" class="emojis">
            <div v-for="item in emojiName()" class="emoji" :key="item" @click="chooseEmoji(item)">
              <img :src="emojiUrl() + emojiMap[item]" style="width: 30px; height: 30px" alt="" />
            </div>
          </div>
          <div v-if="moreType === 'file'" class="file-blocks">
            <input
              type="file"
              id="imagePicker"
              ref="imagePicker"
              accept=".jpg, .jpeg, .png, .gif, .bmp"
              @change="sendImage"
              style="display: none"
            />
            <input type="file" id="filePicker" ref="filePicker" @change="sendFile" style="display: none" />
            <input type="file" id="videoPicker" ref="videoPicker" @change="sendVideo" style="display: none" accept=".mp4, .mov" />
            <div class="block" @click="handleSendImageClick">
              <div class="icon icon-image"></div>
              <span class="name">图片</span>
            </div>
            <div class="block" @click="handleSendVideoClick">
              <div class="icon icon-video"></div>
              <span class="name">视频</span>
            </div>
            <div class="block" @click="handleSendFileClick">
              <div class="icon icon-file"></div>
              <span class="name">文件</span>
            </div>
            <div class="block" @click="handleSendLocationClick">
              <div class="icon icon-location"></div>
              <span class="name">定位</span>
            </div>
          </div>
        </div>
      </div>
    </div>
  </van-popup>
</template>

<script>
import { Popup, NavBar } from "vant";
import MessageItem from "@components/timApp/message/message-item.vue";
import { mapGetters, mapState } from "vuex";
import tim from "@/tim";
import { emojiMap, emojiName, emojiUrl } from "@/utils/emojiMap";

export default {
  name: "current-conversation",
  components: {
    MessageItem,
    [Popup.name]: Popup,
    [NavBar.name]: NavBar,
  },
  props: {
    visible: Boolean,
    defaultMessage: String,
  },
  data() {
    return {
      message: "",
      isShowScrollButtomTips: false,
      preScrollHeight: 0,
      chatType: "text",
      recording: false,
      showMore: false,
      moreType: "",
    };
  },
  computed: {
    emojiMap() {
      return emojiMap;
    },
    ...mapState({
      currentUnreadCount: (state) => state.conversation.currentConversation.unreadCount,
      conversationList: (state) => state.conversation.conversationList.filter((x) => x.type === "C2C"),
      currentConversation: (state) => state.conversation.currentConversation,
      currentMessageList: (state) => state.conversation.currentMessageList,
      currentUserProfile: (state) => state.user.currentUserProfile,
    }),
    ...mapGetters(["toAccount", "currentConversationType", "hidden"]),
    nick() {
      return this.currentConversation.type === "GROUP"
        ? this.currentConversation.groupProfile.name
        : this.currentUserProfile.nick;
    },
    contentHeight() {
      return this.showMore ? `height: calc(100vh - 46px - 60px - 108px);` : ` height: calc(100vh - 46px - 60px);`;
    },
  },
  methods: {
    emojiUrl() {
      return emojiUrl;
    },
    emojiName() {
      return emojiName;
    },
    onClickLeft() {
      this.$emit("update:visible", false);
    },
    send() {
      if (this.message === "" || this.message.trim().length === 0) {
        this.message = "";
        this.$store.commit("showMessage", {
          message: "不能发送空消息哦！",
        });
        return;
      }
      let message = tim.createTextMessage({
        to: this.toAccount,
        conversationType: this.currentConversationType,
        // 消息优先级，用于群聊（v2.4.2起支持）。如果某个群的消息超过了频率限制，后台会优先下发高优先级的消息，详细请参考：https://cloud.tencent.com/document/product/269/3663#.E6.B6.88.E6.81.AF.E4.BC.98.E5.85.88.E7.BA.A7.E4.B8.8E.E9.A2.91.E7.8E.87.E6.8E.A7.E5.88.B6)
        // 支持的枚举值：TIM.TYPES.MSG_PRIORITY_HIGH, TIM.TYPES.MSG_PRIORITY_NORMAL（默认）, TIM.TYPES.MSG_PRIORITY_LOW, TIM.TYPES.MSG_PRIORITY_LOWEST
        // priority: TIM.TYPES.MSG_PRIORITY_NORMAL,
        payload: {
          text: this.message,
        },
        // v2.20.0起支持C2C消息已读回执功能，如果您发消息需要已读回执，需购买旗舰版套餐，并且创建消息时将 needReadReceipt 设置为 true
        needReadReceipt: true,
        // 消息自定义数据（云端保存，会发送到对端，程序卸载重装后还能拉取到，v2.10.2起支持）
        // cloudCustomData: 'your cloud custom data'
      });
      console.log(message);
      // 2. 发送消息
      let promise = tim.sendMessage(message);
      promise
        .then((imResponse) => {
          // 发送成功
          console.log(imResponse);
          this.$store.commit("pushCurrentMessageList", message);
          this.scrollMessageListToBottom();
          this.message = "";
          this.$emit("update:defaultMessage", "");
        })
        .catch((imError) => {
          // 发送失败
          console.warn("sendMessage error:", imError);
          this.$store.commit("showMessage", {
            type: "error",
            message: imError.message,
          });
        });
    },
    // 直接滚到底部
    scrollMessageListToBottom() {
      this.$nextTick(() => {
        let messageListNode = this.$refs["message-list"];
        if (!messageListNode) {
          return;
        }
        messageListNode.scrollTop = messageListNode.scrollHeight;
        // this.preScrollHeight = messageListNode.scrollHeight;
        // this.isShowScrollButtomTips = false;
      });
    },
    switchType(type) {
      console.log(this.currentConversation);
      this.active = type;
    },
    onImageLoaded() {
      this.keepMessageListOnButtom();
    },
    // 如果滚到底部就保持在底部，否则提示是否要滚到底部
    keepMessageListOnButtom() {
      let messageListNode = this.$refs["message-list"];
      if (!messageListNode) {
        return;
      }
      // 距离底部20px内强制滚到底部,否则提示有新消息
      if (this.preScrollHeight - messageListNode.clientHeight - messageListNode.scrollTop < 20) {
        this.$nextTick(() => {
          messageListNode.scrollTop = messageListNode.scrollHeight;
        });
        this.isShowScrollButtomTips = false;
      } else {
        this.isShowScrollButtomTips = true;
      }
      this.preScrollHeight = messageListNode.scrollHeight;
    },
    // 直接滚到底部
    scrollMessageListToButtom() {
      this.$nextTick(() => {
        let messageListNode = this.$refs["message-list"];
        if (!messageListNode) {
          return;
        }
        messageListNode.scrollTop = messageListNode.scrollHeight;
        this.preScrollHeight = messageListNode.scrollHeight;
        this.isShowScrollButtomTips = false;
      });
    },
    startRecording() {
      this.showMore = false;
      this.recording = true;
    },
    endRecording() {
      this.recording = false;
    },
    clickShowMore(type) {
      this.chatType = "text";
      this.moreType = type;
      this.showMore = true;
      this.$bus.$emit("scroll-bottom");
    },
    chooseEmoji(item) {
      this.message += item;
    },
    handleSendImageClick() {
      this.$refs.imagePicker.click();
    },
    handleSendFileClick() {
      this.$refs.filePicker.click();
    },
    handleSendVideoClick() {
      this.$refs.videoPicker.click();
    },
    sendFile() {
      const message = tim.createFileMessage({
        to: this.toAccount,
        conversationType: this.currentConversationType,
        payload: {
          file: document.getElementById("filePicker"), // 或者用event.target
        },
        onProgress: (percent) => {
          this.$set(message, "progress", percent); // 手动给message 实例加个响应式属性: progress
        },
      });
      this.$store.commit("pushCurrentMessageList", message);
      tim
        .sendMessage(message)
        .then(() => {
          // this.$refs.filePicker.value = null;
          document.getElementById("filePicker").value = null;
          this.scrollMessageListToButtom();
        })
        .catch((imError) => {
          this.$store.commit("showMessage", {
            message: imError.message,
            type: "error",
          });
        });
    },
    sendVideo() {
      const message = tim.createVideoMessage({
        to: this.toAccount,
        conversationType: this.currentConversationType,
        payload: {
          file: document.getElementById("videoPicker"), // 或者用event.target
        },
        onProgress: (percent) => {
          this.$set(message, "progress", percent); // 手动给message 实例加个响应式属性: progress
        },
      });
      this.$store.commit("pushCurrentMessageList", message);
      tim
        .sendMessage(message)
        .then(() => {
          // this.$refs.videoPicker.value = null;
          document.getElementById("videoPicker").value = null;
          this.scrollMessageListToButtom();
        })
        .catch((imError) => {
          this.$store.commit("showMessage", {
            message: imError.message,
            type: "error",
          });
        });
    },
    sendImage() {
      const message = tim.createImageMessage({
        to: this.toAccount,
        conversationType: this.currentConversationType,
        payload: {
          file: document.getElementById("imagePicker"), // 或者用event.target
        },
        onProgress: (percent) => {
          this.$set(message, "progress", percent); // 手动给message 实例加个响应式属性: progress
        },
      });
      this.$store.commit("pushCurrentMessageList", message);
      tim
        .sendMessage(message)
        .then(() => {
          // this.$refs.imagePicker.value = null;
          document.getElementById("imagePicker").value = null;
          this.scrollMessageListToButtom();
        })
        .catch((imError) => {
          this.$store.commit("showMessage", {
            message: imError.message,
            type: "error",
          });
        });
    },
    // HTML5 Geolocation（地理定位）用于定位用户的位置。

    // Internet Explorer 9+, Firefox, Chrome, Safari 和 Opera 支持Geolocation（地理定位）.
    // 注意: Geolocation（地理定位）对于拥有 GPS 的设备，比如 iPhone，地理定位更加精确。
    navigatorLocation() {
      return new Promise((resolve, reject) => {
        if (navigator.geolocation) {
          // alert("该浏览器支持获取地理位置。")
          navigator.geolocation.getCurrentPosition(
            (position) => {
              let longitude = position.coords.longitude;
              let latitude = position.coords.latitude;
              console.log(longitude, latitude);
              resolve({ longitude, latitude });
            },
            (error) => {
              let msg;
              switch (error.code) {
                case 1:
                  msg = "用户拒绝对获取地理位置的请求。";
                  break;
                case 2:
                  msg = "位置信息是不可用的。";
                  break;
                case 3:
                  msg = "请求用户地理位置超时。";
                  break;
                case 4:
                  msg = "未知错误。";
                  break;
              }
              this.$store.commit("showMessage", {
                message: msg,
                type: "error",
              });
              reject();
              // alert("获取地理位置失败。" + msg);
            }
          );
        } else {
          this.$store.commit("showMessage", {
            message: "该浏览器不支持获取地理位置。",
            type: "error",
          });
          reject();
          // alert("该浏览器不支持获取地理位置。");
        }
      });
    },
    async handleSendLocationClick() {
      // 发送地理位置消息，Web 端与小程序端相同
      // 1. 创建消息实例，接口返回的实例可以上屏
      try {
        const position = await this.navigatorLocation();
        console.log(position);
        let { longitude, latitude } = position;
        // console.log(longitude, latitude);
        // 1. 创建消息实例，接口返回的实例可以上屏
        let message = tim.createLocationMessage({
          to: this.toAccount,
          conversationType: this.currentConversationType,
          // 消息优先级，用于群聊。如果某个群的消息超过了频率限制，后台会优先下发高优先级的消息，详细请参考：https://cloud.tencent.com/document/product/269/3663#.E6.B6.88.E6.81.AF.E4.BC.98.E5.85.88.E7.BA.A7.E4.B8.8E.E9.A2.91.E7.8E.87.E6.8E.A7.E5.88.B6)
          // 支持的枚举值：TencentCloudChat.TYPES.MSG_PRIORITY_HIGH, TencentCloudChat.TYPES.MSG_PRIORITY_NORMAL（默认）, TencentCloudChat.TYPES.MSG_PRIORITY_LOW, TencentCloudChat.TYPES.MSG_PRIORITY_LOWEST
          // priority: TencentCloudChat.TYPES.MSG_PRIORITY_NORMAL,
          payload: {
            description: "安全保障",
            longitude: longitude, // 经度
            latitude: latitude, // 纬度
          },
        });
        // 2. 发送消息
        this.$store.commit("pushCurrentMessageList", message);
        tim
          .sendMessage(message)
          .then(() => {
            this.scrollMessageListToButtom();
          })
          .catch((imError) => {
            this.$store.commit("showMessage", {
              message: imError.message,
              type: "error",
            });
          });
      } catch (err) {
        console.error(err.message);
      }
    },
  },
  mounted() {
    this.$bus.$on("image-loaded", this.onImageLoaded);
    this.$bus.$on("scroll-bottom", this.scrollMessageListToButtom);
    if (this.defaultMessage) {
      this.message = this.defaultMessage;
      this.send();
    }
    // this.$nextTick(() => {
    //   this.scrollMessageListToButtom();
    // });
  },
};
</script>

<style scoped lang="less">
.main {
  width: 100%;
  height: calc(100vh - 46px);
  .conversation {
    width: 100%;
    .content {
      width: 100%;
      //height: calc(100vh - 46px - 60px);
      min-height: 200px;
      overflow-y: auto;
      overflow-x: hidden;
      padding: 0 24px;
      background: #f4f5f7;
      box-sizing: border-box;
    }
    .footer {
      width: 100%;
      height: 60px;
      background: #ffffff;
      display: flex;
      align-items: center;
      padding: 18px;
      box-sizing: border-box;
      .chat-input {
        width: 231px;
        height: 40px;
        background: #f1f5fa;
        border-radius: 6px 6px 6px 6px;
        font-size: 18px;
        font-family: PingFang SC-Regular, PingFang SC;
        font-weight: 400;
        line-height: 20px;
        box-sizing: border-box;
        border: 0;
        padding: 10px 8px;
        resize: none;
        margin-left: 12px;
        &::placeholder {
          color: #c0c4cc;
        }
      }
      .voice-block {
        width: 231px;
        height: 40px;
        background: #f1f5fa;
        border-radius: 6px 6px 6px 6px;
        font-size: 18px;
        font-family: PingFang SC-Medium, PingFang SC;
        font-weight: 500;
        color: #303133;
        line-height: 20px;
        display: flex;
        align-items: center;
        justify-content: center;
        user-select: none;
        margin-left: 12px;
      }
      .chat-type {
        display: block;
        width: 24px;
        height: 24px;
        &-text {
          background: url("~@/assets/images/conversation/键盘.png") no-repeat;
          background-size: 100% 100%;
        }
        &-voice {
          background: url("~@/assets/images/conversation/语音.png") no-repeat;
          background-size: 100% 100%;
        }
      }
      .btn {
        width: 24px;
        height: 24px;
        display: block;
        margin-left: 12px;
        &-more {
          background: url("~@/assets/images/conversation/更多.png") no-repeat;
          background-size: 100% 100%;
        }
        &-emoji {
          background: url("~@/assets/images/conversation/emoji.jpg") no-repeat;
          background-size: 100% 100%;
        }
      }
      .toolbar {
        width: 100%;
        height: 65px;
        display: flex;
        align-items: center;
        padding-left: 24px;
        .icon-smile {
          display: block;
          width: 24px;
          height: 24px;
          background: url("~@/assets/images/conversation/smile-outlined.png") no-repeat;
          background-size: 100% 100%;
          cursor: pointer;
        }
        .icon-folder {
          display: block;
          width: 24px;
          height: 24px;
          background: url("~@/assets/images/conversation/folder-outlined.png") no-repeat;
          background-size: 100% 100%;
          cursor: pointer;
          margin-left: 16px;
        }
      }
      ::v-deep .modal {
        width: calc(100% - 24px - 120px - 24px);
        height: calc(100% - 56px - 8px);
        .el-textarea__inner {
          border: 0;
        }
      }
      .send {
        width: 120px;
        height: 48px;
        margin-left: 24px;
      }
    }
    .under-footer {
      width: 100%;
      height: 108px;
      background: #ffffff;
      padding: 14px 17px 12px 17px;
      box-sizing: border-box;
      .file-blocks {
        width: 100%;
        height: 100%;
        box-sizing: border-box;
        display: flex;
        flex-direction: row;
        flex-wrap: wrap;
        overflow-y: scroll;
        .block {
          width: 60px;
          height: 82px;
          display: flex;
          flex-direction: column;
          align-items: center;
          justify-content: space-between;
          margin-right: 30px;
          .icon {
            width: 60px;
            height: 60px;
            &-image {
              background: url("~@/assets/images/conversation/image.png") no-repeat;
              background-size: 100% 100%;
            }
            &-file {
              background: url("~@/assets/images/conversation/file.png") no-repeat;
              background-size: 100% 100%;
            }
            &-video {
              background: url("~@/assets/images/conversation/video.png") no-repeat;
              background-size: 100% 100%;
            }
            &-location {
              background: url("~@/assets/images/conversation/location.png") no-repeat;
              background-size: 100% 100%;
            }
          }
          .name {
            font-size: 14px;
            font-family: PingFang SC-Regular, PingFang SC;
            font-weight: 400;
            color: #6f737f;
            line-height: 14px;
          }
        }
      }
    }
  }
}
</style>
